package dougw.unc.edu.webfeeler;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import dougw.unc.edu.webfeeler.WebfeelerUtil;
import dougw.unc.edu.webfeeler.WebfeelerConstants;
import dougw.unc.edu.webfeeler.WebfeelerLogger;

import uk.ac.umist.co.brailletrans.LanguageUnicode;

public class Webfeeler {
	
	// ----OBJECT DATA
	private String FILE_SEPARATOR;
	private String LINE_SEPARATOR;
	private String CLASSPATH;
	private String CLASSLOC;
	
	private LanguageUnicode translator;
	private String lang;
	private int state;
	private boolean	good;
	
	private static String constructorOut;
    
	// Constructor
    public Webfeeler() {
    	this.good = true; // this represents if the WF constructor was complete
    	/*
    	 * Record the proper directory separator for 
    	 * differing operating systems.
    	 */ 
    	LINE_SEPARATOR = "\n";
    	constructorOut = "Constructor Messages:"+LINE_SEPARATOR;
    	constructorOut += "Line Separator: "+LINE_SEPARATOR;
    	FILE_SEPARATOR = System.getProperty("file.separator");
    	constructorOut += "File Separator: "+FILE_SEPARATOR+LINE_SEPARATOR;
    	this.CLASSPATH = System.getProperty("java.class.path");
    	constructorOut += "Classpath: "+this.CLASSPATH+LINE_SEPARATOR;
    	this.CLASSLOC = WebfeelerUtil.getDiskLocation(Webfeeler.class, FILE_SEPARATOR);
    	System.setProperty("user.dir", this.CLASSLOC);
    	constructorOut += "Working directory: "+System.getProperty("user.dir")+LINE_SEPARATOR;
    	System.setProperty("java.library.path", CLASSLOC+";"+System.getProperty("java.library.path"));
    	constructorOut += "Java library path: "+System.getProperty("java.library.path")+LINE_SEPARATOR;

    	
		/*
		 * Create the translator object
		 * From package import uk.ac.umist.co.brailletrans;
		 * http://www.alasdairking.me.uk/brailletrans/
		 */
    	try {
    		this.lang = WebfeelerConstants.DEF_LANGUAGE;
			this.translator = new LanguageUnicode(this.lang);
	    	constructorOut += "Instantiation of LanguageUnicode object with lang: "+this.lang+LINE_SEPARATOR;
			this.state = WebfeelerConstants.DEF_STATE;
	    	this.translator.setState(this.state);
	    	constructorOut += "Translator State: "+this.translator.getState()+LINE_SEPARATOR;
			
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			this.good = false;
	    	constructorOut += "LanguageUnicode threw: "+e.toString()+LINE_SEPARATOR;
		} catch (IOException e) {
			e.printStackTrace();
			this.good = false;
	    	constructorOut += "LanguageUnicode threw: "+e.toString()+LINE_SEPARATOR;
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			this.good = false;
	    	constructorOut += "LanguageUnicode threw: "+e.toString()+LINE_SEPARATOR;
		}		
    }
    
    public String save(String URL, String dest, String filePrefix, String fileSuffix, int state) {
    	if(!this.good)
    		return WebfeelerConstants.ERR_WF_BAD;
    	
    	
    	if(this.state != state) {
    		setState(state);
    	}
    	
    	// prepare the destination directory and fix trailing slashes
        String dirPath = WebfeelerUtil.cleanDirPath(dest, FILE_SEPARATOR);
        if(isError(dirPath)) return dirPath;
        // get a unique file name
        File destFile = WebfeelerUtil.getUniqueFilename(dirPath, filePrefix, fileSuffix);
    	
    	System.out.println("Attempting to save "+URL+" to "+destFile.toString());
    	String result = WebfeelerUtil.cacheURL(URL);
    	if(isError(result)) return result;

    	String braille;
    	try {
			braille = this.transcribe(WebfeelerUtil.getTextFromHTML(result));
		} catch (Exception e) {
			e.printStackTrace();
			return WebfeelerConstants.ERR_TRS_TRS + " " + e.toString();
		} 
    	
    	if(WebfeelerUtil.writeToDisk(destFile, WebfeelerUtil.cleanText(braille))) {
    		return destFile.toString();
    	}
    	else {
    		return WebfeelerConstants.ERR_WTD_WTD+" "+destFile.toString();
    	}
    }
    
    public String print(String URL, String port, int state) {
    	if(!this.good)
    		return WebfeelerConstants.ERR_WF_BAD;
    	
    	if(this.state != state) {
    		setState(state);
    	}
    	WebfeelerLogger.log("Attempting to print "+URL);
    	String result = WebfeelerUtil.cacheURL(URL);
    	if(isError(result)) return result;
    	String braille;
    	try {
			braille = this.transcribe(WebfeelerUtil.getTextFromHTML(result));
		} catch (Exception e) {
			e.printStackTrace();
			return WebfeelerConstants.ERR_TRS_TRS + " " + e.toString();
		}
		
		result = WebfeelerUtil.toEmbosser(WebfeelerUtil.cleanText(braille), port);
		if(isError(result)) return result;
		return port;
    }
    
    
    public String transcribe(String s) {   
    	int length = s.length();
    	int next = 0, caret = 0;
    	String finished = "";
    	do {
    		next = s.indexOf(LINE_SEPARATOR,caret+1);
    		if(next == -1)
    			caret = next = length;
    		finished += this.translator.translate(s.substring(caret,next))+LINE_SEPARATOR;
    		caret = next+1;
    	} while (caret < length);
    	System.out.println(finished);
    	return finished;
    } 
    
    public void setState(int state) {
    	this.state = state;
    	this.translator.setState(state);
    }
    
    public void setLanguage(String lang) {
    	this.lang = lang;
    	try {
			/*
			 * Create the translator object
			 * From package import uk.ac.umist.co.brailletrans;
			 * http://www.alasdairking.me.uk/brailletrans/
			 */
			this.translator = new LanguageUnicode(lang);
	    	constructorOut += ".setLanguage(): New Instantiation of LanguageUnicode object with lang: "+lang+LINE_SEPARATOR;
			this.translator.setState(this.state);
	    	constructorOut += "Translator State: "+this.translator.getState()+LINE_SEPARATOR;
			
		} catch (FileNotFoundException e) {
			e.printStackTrace();
	    	constructorOut += "LanguageUnicode threw: "+e.toString()+LINE_SEPARATOR;
		} catch (IOException e) {
			e.printStackTrace();
	    	constructorOut += "LanguageUnicode threw: "+e.toString()+LINE_SEPARATOR;
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
	    	constructorOut += "LanguageUnicode threw: "+e.toString()+LINE_SEPARATOR;
		}
    }
    
    private boolean isError(String s) {
    	WebfeelerLogger.debug("Testing for error string: "+s);
    	if(s.startsWith(WebfeelerConstants.ERR_START)) {
    		WebfeelerLogger.debug("Error: "+s);
    		return true;
    		
    	}
    	return false;
    }
    
    public String getClassLocation() {
    	return this.CLASSLOC;
    }
    
    static public String getConstructorOutput() {
    	return constructorOut;
    }
    
    // this is used to show that the class was successfully loaded with 
    // the class loader.  This message is traced to the javascript console
    static public String getGreeting() {
        return WebfeelerConstants.G_INIT_SUCCESS;
    }
}
