package dougw.unc.edu.webfeeler;


import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.net.URI;
import java.net.URL;
import java.util.Iterator;

import dougw.unc.edu.webfeeler.WebfeelerConstants;

import dougw.unc.edu.util.domWalker.ExtractText;
import dougw.unc.edu.util.domWalker.FileList;
import dougw.unc.edu.util.domWalker.InputUni;

/**
 * @author dfhuynh
 */
public class WebfeelerUtil {
	

	// write string to disk
	// Pre: assumes that outfile is the fill path
    public static boolean writeToDisk(File outFile, String toWrite) {	
    	try {
            BufferedWriter out = new BufferedWriter(new FileWriter(outFile));
            out.write(toWrite);
            out.close();
        } catch (IOException e) {
        	e.printStackTrace();
        	return false;
        }
    	return true;
    }
    
    public static String cacheURL(String inURL) {
        InputStream inputStream = null;
        DataInputStream dataInputStream;
        String s;
        String contents = null;
        URL url;
        try {
        	url = new URL(inURL);
        } catch (MalformedURLException e) {
            e.printStackTrace();
            return WebfeelerConstants.ERR_WTD_MUE + " " + e.toString();
        } 
        try {
           inputStream = url.openStream();
           dataInputStream = new DataInputStream(new BufferedInputStream(inputStream));
           while ((s = dataInputStream.readLine()) != null) {
              contents += s;
           }
           //         Create a tempoary file to hold the contents
           File temp = null;
           try {
               temp = File.createTempFile(WebfeelerConstants.DEF_FILE_PREFIX, WebfeelerConstants.DEF_FILE_SUFFIX);
               temp.deleteOnExit();
           
               // Write to temp file
               BufferedWriter out = new BufferedWriter(new FileWriter(temp));
               out.write(contents);
               out.close();
               inputStream.close();
           } catch (IOException e) {
           		e.printStackTrace();
           		return WebfeelerConstants.ERR_WTD_IOE.replaceAll("URL", inURL);
           }
           	WebfeelerLogger.debug("Cache file: "+temp.toString());
   			return temp.toString();
        } catch (NullPointerException e) {
            e.printStackTrace();
            return WebfeelerConstants.ERR_WTD_NPE.replaceAll("URL", inURL);
        } catch (UnknownHostException e) {
            e.printStackTrace();
            return WebfeelerConstants.ERR_WTD_UHE.replaceAll("URL", inURL);
        } catch (IOException e) {
            e.printStackTrace();
            return WebfeelerConstants.ERR_WTD_IOE.replaceAll("URL", inURL);
        }
        
        
        
    }
    
    public static String getTextFromHTML(String inFile) {
        ExtractText extracttext = new ExtractText();
        String[] args = {inFile};
        String contents = "";
        int i = extracttext.commandLine(args);
        URI uri;
		try {
			uri = (new File(".")).getCanonicalFile().toURI();
		} catch (IOException e) {
			e.printStackTrace();
			return WebfeelerConstants.ERR_EXT_URI; // Return URI Error
		}
		Iterator iterator = (new FileList(inFile, null)).iterator();
		try {
			Object obj = iterator.next();
				contents = extracttext.extract(InputUni.getInstance(obj, null, null));
		}
		catch(Exception e) {
				e.printStackTrace();
				return WebfeelerConstants.ERR_EXT_EXT;
		}
		WebfeelerLogger.debug("Text from HTML file: "+inFile.toString()+"\n" + contents);
		return contents;
    }
    
    public static String cleanText(String s) {
    	/*
    	 * Hack around debuging for extract text
	 * clean this up.
    	 * 
    	 */
    		String lineOne = "";
    		String lineTwo = "";
    		String lineThree = "";
    		String contents = s;
    		int caret = 0;
    		int counter = 1;
    		try {
    			 		lineOne = s.substring(caret, s.indexOf("\n", caret));
    			 		caret = s.indexOf("\n", caret)+1;
    			 		lineTwo = s.substring(caret, s.indexOf("\n", caret));
    			 		caret = s.indexOf("\n", caret)+1;
    			 		lineThree = s.substring(caret, s.indexOf("\n", caret));
    			 		caret = s.indexOf("\n", caret)+1;
    			 		contents = s.substring(caret);
    			 		contents = lineTwo+"\n\n"+contents;
    			} catch (StringIndexOutOfBoundsException e) {
    				return s;
    			}
    		return contents;
    			
    }
    
    
    public static String getDiskLocation(Class c, String fileSeparator) {
    	try {
    		URL url = c.getProtectionDomain().getCodeSource().getLocation();
    		String path = url.toString();
    		if (path != null && path.startsWith("file")) {
    			File file = new File(url.getFile());
    			return file.getAbsolutePath().substring(0,file.getAbsolutePath().lastIndexOf(fileSeparator));
    		} else {
    			return WebfeelerConstants.ERR_LOC_UTD;
    		}
    	} catch (Throwable t) {
    		return WebfeelerConstants.ERR_LOC_UTD;
    	}
    }
    
    public static String toEmbosser(String text, String port) {
    	String newline = "0x0D";
    	
    	try {
  	      BufferedWriter printer = new BufferedWriter(new FileWriter(port), 512);
  	      printer.write(text);
  	      printer.close();
    	}
    	catch (IOException e) { // Embosser IO
    	      e.printStackTrace();
    	      return WebfeelerConstants.ERR_EMB_EIO;
    	}
		return port;
    }
    
    // return a directory path with one and only one fileSeparator at the end
    // eg C:\dir\temp\
    public static String cleanDirPath(String dirPath, String fileSeparator) {
    	// prepare the destination directory and fix trailing slashes
    	while(dirPath.endsWith(fileSeparator)) // clear the trailing slashs from test
    		dirPath = dirPath.substring(0,dirPath.length()-1);
    	dirPath += fileSeparator;  			// add ONE trailling slash
    	return dirPath;
    }
    
    // return the full path to a unique filename
    // add (int) if neccessary
    // pre: dirPath must be clean and contain only one file separator at the end
    public static File getUniqueFilename(String dirPath, String filePrefix, String fileSuffix) {
    	// Find a filename that is not in use and unique
    	int i = 0;
    	File destFile = new File(dirPath+filePrefix+fileSuffix);
    	while ( destFile.exists() ) {
    		i++;
    		destFile = new File(dirPath+filePrefix+"("+i+")"+fileSuffix);
    	}
    	return destFile;
    }
}
